@@ -27,16 +27,19 @@
 
 import pytest
 
-from libqtile import layout
-import libqtile.manager
 import libqtile.config
-from .layout_utils import assertDimensions
-from ..conftest import no_xinerama
+from libqtile import layout
+from libqtile.confreader import Config
+from test.conftest import no_xinerama
+from test.layouts.layout_utils import (
+    assert_dimensions,
+    assert_focus_path,
+    assert_focused,
+)
 
 
-class VerticalTileConfig(object):
+class VerticalTileConfig(Config):
     auto_fullscreen = True
-    main = None
     groups = [
         libqtile.config.Group("a"),
         libqtile.config.Group("b"),
@@ -46,32 +49,52 @@ class VerticalTileConfig(object):
     layouts = [
         layout.VerticalTile(columns=2)
     ]
-    floating_layout = libqtile.layout.floating.Floating()
+    floating_layout = libqtile.resources.default_config.floating_layout
     keys = []
     mouse = []
     screens = []
 
 
-verticaltile_config = lambda x: \
-    no_xinerama(pytest.mark.parametrize("qtile", [VerticalTileConfig], indirect=True)(x))
+def verticaltile_config(x):
+    return no_xinerama(pytest.mark.parametrize("manager", [VerticalTileConfig], indirect=True)(x))
 
 
 @verticaltile_config
-def test_verticaltile_simple(qtile):
-    qtile.testWindow("one")
-    assertDimensions(qtile, 0, 0, 800, 600)
-    qtile.testWindow("two")
-    assertDimensions(qtile, 0, 300, 798, 298)
-    qtile.testWindow("three")
-    assertDimensions(qtile, 0, 400, 798, 198)
+def test_verticaltile_simple(manager):
+    manager.test_window("one")
+    assert_dimensions(manager, 0, 0, 800, 600)
+    manager.test_window("two")
+    assert_dimensions(manager, 0, 300, 798, 298)
+    manager.test_window("three")
+    assert_dimensions(manager, 0, 400, 798, 198)
 
 
 @verticaltile_config
-def test_verticaltile_maximize(qtile):
-    qtile.testWindow("one")
-    assertDimensions(qtile, 0, 0, 800, 600)
-    qtile.testWindow("two")
-    assertDimensions(qtile, 0, 300, 798, 298)
+def test_verticaltile_maximize(manager):
+    manager.test_window("one")
+    assert_dimensions(manager, 0, 0, 800, 600)
+    manager.test_window("two")
+    assert_dimensions(manager, 0, 300, 798, 298)
     # Maximize the bottom layout, taking 75% of space
-    qtile.c.layout.maximize()
-    assertDimensions(qtile, 0, 150, 798, 448)
+    manager.c.layout.maximize()
+    assert_dimensions(manager, 0, 150, 798, 448)
+
+
+@verticaltile_config
+def test_verticaltile_window_focus_cycle(manager):
+    # setup 3 tiled and two floating clients
+    manager.test_window("one")
+    manager.test_window("two")
+    manager.test_window("float1")
+    manager.c.window.toggle_floating()
+    manager.test_window("float2")
+    manager.c.window.toggle_floating()
+    manager.test_window("three")
+
+    # test preconditions
+    assert manager.c.layout.info()['clients'] == ['one', 'two', 'three']
+    # last added window has focus
+    assert_focused(manager, "three")
+
+    # assert window focus cycle, according to order in layout
+    assert_focus_path(manager, 'float1', 'float2', 'one', 'two', 'three')
