@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # Copyright (c) 2011 Florian Mounier
 # Copyright (c) 2014 Sean Vig
 #
@@ -22,23 +22,16 @@
 """
     This program is carefully crafted to exercise a number of corner-cases in
     Qtile.
+
+    Usage:
+        window.py DISPLAY WINDOW_NAME [WINDOW_TYPE]
 """
-from __future__ import print_function
 import sys
 import time
+
 import xcffib
 import xcffib.xproto
 
-
-def configure(window):
-    window.configure(
-        width=100,
-        height=100,
-        x=0,
-        y=0,
-        border_width=1,
-    )
-
 for i in range(20):
     try:
         conn = xcffib.connect(display=sys.argv[1])
@@ -79,6 +72,16 @@ conn.core.ChangeProperty(xcffib.xproto.PropMode.Replace,
                          xcffib.xproto.Atom.ATOM, 32, 1,
                          [wm_delete_window])
 
+if len(sys.argv) > 3:
+    wm_type_atom = "_NET_WM_WINDOW_TYPE"
+    wm_type_atom = conn.core.InternAtom(0, len(wm_type_atom), wm_type_atom).reply().atom
+    wm_type = "_NET_WM_WINDOW_TYPE_" + sys.argv[3].upper()
+    wm_type = conn.core.InternAtom(0, len(wm_type), wm_type).reply().atom
+    conn.core.ChangeProperty(xcffib.xproto.PropMode.Replace,
+                             window, wm_type_atom,
+                             xcffib.xproto.Atom.ATOM, 32, 1,
+                             [wm_type])
+
 conn.core.ConfigureWindow(window,
                           xcffib.xproto.ConfigWindow.X | xcffib.xproto.ConfigWindow.Y |
                           xcffib.xproto.ConfigWindow.Width | xcffib.xproto.ConfigWindow.Height |
